import unittest
import numpy as np

import timeit
from escape.core import *


class TestMathMethods(unittest.TestCase):
     
    def test_parameter_math(self):
        pass
     
    def test_functor_math(self):
        pass
     
    def test_integration_over_variable(self):
        x = var("x")
        f = x * exp(-x * x)
        fi = integral(f, x, 0, 1, epsabs=1e-4, epsrel=1e-5, maxiter=1)
        self.assertAlmostEqual(fi.value, -0.5 * (np.exp(-1) - 1.0))
     
    def test_integration_over_parameter(self):
        x = var("x")
        p = par("P", 1.0, userlim=[0, 1])
        f = x * exp(-x * x * p)
          
        def res(x):
            return -1.0 / x * (np.exp(-x * x) - 1.0)
          
        fi = integral(f, p, 0, 1, epsabs=1e-4, epsrel=1e-5, maxiter=10)
        for i in range(1, 10):
            self.assertAlmostEqual(fi(i), res(i))
              
    def test_integration_over_parameter2(self):
        x = var("x")
        p = par("P", 1.0, userlim=[0, 1])
        f = sin(p * x)
          
        def res(x):
            return -1.0 / x * (np.cos(x) - 1.0)
          
        fi = integral(f, p, 0, 1, epsabs=1e-4, epsrel=1e-5, maxiter=10)
        for i in range(1, 10):
            self.assertAlmostEqual(fi(i), res(i))
      
    def test_average_over_variable(self):
        
        x = var("x")
        x0 = var("x0")
        fwhm = func("", x0, 1.0)
        
        dgauss = normal("normal", x, x0, fwhm)
        
        
        
        cv = average(x, dgauss, x, x0, x0 - 3* 2.12 * fwhm, x0 + 3* 2.12 * fwhm, epsabs=1e-5, maxiter=10)
   
        absc = np.linspace(0.0, 200.0, 1000)
           
        for i in range(len(absc)):
            _x = absc[i]
            self.assertAlmostEqual(cv(_x), _x, delta=1e-5)
       
    def test_average_over_parameter(self):
        pass
       
    def test_normal_resolution(self):
        x = var("x")
        x0 = var("x0")
        fwhm = func("", x0, 1.0)
        
        cv = average_normal(x, fwhm, x, x0, storage_size=15, epsabs=1e-5, maxiter=5, numstd=7)
   
        absc = np.linspace(0.0, 200.0, 200)
           
        for i in range(len(absc)):
            _x = absc[i]
            self.assertAlmostEqual(cv(_x), _x, delta=1e-5)
            
    def test_uniform_resolution(self):
        x = var("x")
        x0 = var("x0")
        fwhm = func("", x0, 1.0)
        
        cv = average_uniform(x, fwhm, x, x0, storage_size=15, epsabs=1e-5, maxiter=5)
   
        absc = np.linspace(0.0, 200.0, 200)
           
        for i in range(len(absc)):
            _x = absc[i]
            self.assertAlmostEqual(cv(_x), _x, delta=1e-5)
            
    def test_triangular_resolution(self):
        x = var("x")
        x0 = var("x0")
        fwhm = func("", x0, 1.0)
        
        cv = average_triangular(x, fwhm, x, x0, storage_size=15, epsabs=1e-5, maxiter=5)
   
        absc = np.linspace(0.0, 200.0, 200)
           
        for i in range(len(absc)):
            _x = absc[i]
            self.assertAlmostEqual(cv(_x), _x, delta=1e-5)


if __name__ == '__main__':
    unittest.main()
